/*
 *  graph.h - helpful graph structure and setup/teardown methods
 *
 *  Copyright (c) 2007-2019 Pacman Development Team <pacman-dev@archlinux.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef HBPM_GRAPH_H
#define HBPM_GRAPH_H

#include <sys/types.h> /* off_t */

#include "hbpm_list.h"

enum __hbpm_graph_vertex_state {
	HBPM_GRAPH_STATE_UNPROCESSED,
	HBPM_GRAPH_STATE_PROCESSING,
	HBPM_GRAPH_STATE_PROCESSED
};

typedef struct __hbpm_graph_t {
	void *data;
	struct __hbpm_graph_t *parent; /* where did we come from? */
	hbpm_list_t *children;
	hbpm_list_t *iterator; /* used for DFS without recursion */
	off_t weight; /* weight of the node */
	enum __hbpm_graph_vertex_state state;
} hbpm_graph_t;

hbpm_graph_t *_hbpm_graph_new(void);
void _hbpm_graph_free(void *data);

#endif /* HBPM_GRAPH_H */
