/*
 *  pkghash.h
 *
 *  Copyright (c) 2011-2019 Pacman Development Team <pacman-dev@archlinux.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef HBPM_PKGHASH_H
#define HBPM_PKGHASH_H

#include <stdlib.h>

#include "hbpm.h"
#include "hbpm_list.h"


/**
 * @brief A hash table for holding hbpm_pkg_t objects.
 *
 * A combination of a hash table and a list, allowing for fast look-up
 * by package name but also iteration over the packages.
 */
struct __hbpm_pkghash_t {
	/** data held by the hash table */
	hbpm_list_t **hash_table;
	/** head node of the hash table data in normal list format */
	hbpm_list_t *list;
	/** number of buckets in hash table */
	unsigned int buckets;
	/** number of entries in hash table */
	unsigned int entries;
	/** max number of entries before a resize is needed */
	unsigned int limit;
};

typedef struct __hbpm_pkghash_t hbpm_pkghash_t;

hbpm_pkghash_t *_hbpm_pkghash_create(unsigned int size);

hbpm_pkghash_t *_hbpm_pkghash_add(hbpm_pkghash_t **hash, hbpm_pkg_t *pkg);
hbpm_pkghash_t *_hbpm_pkghash_add_sorted(hbpm_pkghash_t **hash, hbpm_pkg_t *pkg);
hbpm_pkghash_t *_hbpm_pkghash_remove(hbpm_pkghash_t *hash, hbpm_pkg_t *pkg, hbpm_pkg_t **data);

void _hbpm_pkghash_free(hbpm_pkghash_t *hash);

hbpm_pkg_t *_hbpm_pkghash_find(hbpm_pkghash_t *hash, const char *name);

#endif /* HBPM_PKGHASH_H */
